(* Turbo51-Unit
  interrupt buffered reading from serial port (standard in)
  pure Pascal version

  Important:  Main program must contain directive "{$DefaultFile On }"
  ==========
              requires extended memory
   
  J. Rathlev, Jan. 2011
  *)

unit SerialInOutPas;

{$XDATA}

// ----------------------------------------------------------------- 
interface 

{$I CpuClock.inc}

const
  DefBaudRate = 9600;
  BufLen = 256;   // length of input buffer
  Lf = 10;
  Cr = 13;

function KeyPressed : boolean;
function ReadKey : char; 

procedure SetBaudRate (Baudrate : longint);

procedure WriteToSerialPort;
function ReadFromSerialPort: Char;
function ReceivedChars : byte;
function ReceivedLines: byte;

procedure SerialInt; interrupt Serial; using 3;
 
// ----------------------------------------------------------------- 
implementation

var
  BaudRateTimerValue : Byte;
  ICCnt,ILCnt,
  IWPnt,IRPnt : word; volatile;
  OutRdy : boolean; volatile;
  InBuf : array [0..BufLen-1] of byte XDATA; volatile;

// ----------------------------------------------------------------- 
// change chracter to upcase (only standard ASCII)
function UpCase (c : char) : char;
begin
  if (c>='a') and (c<='z') then Result:=char(byte(c) and $DF);
  end;
  
// ----------------------------------------------------------------- 
// interrupt procedure receive character from serial port
procedure SerialInt;
var
  c : byte;
begin
  if RI then begin
    RI:=false;
    c:=SBUF;
    P1:=not c;
    if c=Lf then Exit;             // ignore line feed
    if ICCnt>=BufLen then Exit;    // buffer overflow - ignore char
    if c=CR then inc(ILCnt);       // count lines
    InBuf[IWPnt]:=c;
    inc(IWPnt);
    if IWPnt>=BufLen then IWPnt:=0;
    inc(ICCnt);
    end;
  if TI then begin
    TI:=false;
    OutRdy:=true;
    end;
  end;
  
// ----------------------------------------------------------------- 
// Basic I/O routines for "Assign"
// send character to serial port (polling)
procedure WriteToSerialPort; Assembler;
Asm
@WaitLoop:
  JNB       OutRdy, @WaitLoop  // wait for TI cleared by interrupt
  MOV       SBUF, A 
  CLR       OutRdy
  end;

// receive character from  serial port (via interrupt)
{$O-}
function ReadFromSerialPort: Char;
var 
  ByteResult: Byte absolute Result;
begin
  Asm  
//    PUSH      DPL
//    PUSH      DPH
    PUSH      AR2
    PUSH      AR3
    end;
  repeat until ICCnt>0;
  dec(ICCnt);
  ByteResult:=InBuf[IRPnt];
  inc(IRPnt);
  if IRPnt>=BufLen then IRPnt:=0;
  if ByteResult=Cr then begin
    if ILCnt>0 then dec(ILCnt);
    ByteResult:=Lf;
    end;
  Asm
    POP       AR3
    POP       AR2
//    POP       DPH
//    POP       DPL
    end;
  end;
{$O+}

// ----------------------------------------------------------------- 
// number of characters in input buffer
function ReceivedChars : byte;
begin
  Result:=ICCnt;
  end;

// number of lines in input buffer
function ReceivedLines: byte;
begin
  Result:=ILCnt;
  end;

// test if at least one character in input buffer
function KeyPressed : boolean;
begin
  Result:=ICCnt>0;
  end;

// read one character from input buffer
function ReadKey : char; 
begin
  Result:=ReadFromSerialPort;
  end;
    
// set baud rate for serial port
procedure SetBaudRate (Baudrate : longint);
begin
  BaudRateTimerValue:=256-CpuClock div (12*16*Baudrate);
  TL1  := BaudRateTimerValue;
  TH1  := BaudRateTimerValue;
  end;
  
// initialize serial port    
procedure InitSerial;
begin
  SetBaudrate(defBaudRate);
  TMOD := TMOD and $F or %00100000;    { Timer1: no GATE, 8 bit timer, autoreload }
  SCON := %01010000;    { Serial Mode 1, Enable Reception }
  PCON := PCON or $80;
  TR1  := True;         { Enable timer 1 }
  TI   := True;         { Indicate TX ready }
  ICCnt:=0; ILCnt:=0; IWPnt:=0; IRPnt:=0;
  ES:=true;  {Enable serial interrupt}
  end;

begin
  InitSerial;
  Assign (CurrentIO, ReadFromSerialPort, WriteToSerialPort);
end.